@@ -1,5 +1,5 @@
 package Form::Tiny::Utils;
-$Form::Tiny::Utils::VERSION = '2.19';
+$Form::Tiny::Utils::VERSION = '2.20';
 use v5.10;
 use strict;
 use warnings;
@@ -148,16 +148,13 @@ sub _find_field
 			push @found, [$curr_path, $value];
 		}
 		else {
-			my $is_array = $arrays[$index];
 			my $current_ref = ref $value;
 
-			# make sure the actual ref type does not mismatch the spec
-			return unless $is_array eq ($current_ref eq 'ARRAY');
+			if ($arrays[$index]) {
 
-			# it's not the leaf of the tree yet, so we require an array or a hash
-			return if !$is_array && $current_ref ne 'HASH';
+				# It's an array, make sure the actual ref type does not mismatch the spec
+				return unless $current_ref eq 'ARRAY';
 
-			if ($is_array) {
 				if (@$value == 0) {
 
 					# we wanted to have a deeper structure, but its not there, so clearly an error
@@ -175,8 +172,9 @@ sub _find_field
 			}
 
 			else {
+				# it's not the leaf of the tree yet, so we require a hash
 				my $next = $path[$index];
-				return unless exists $value->{$next};
+				return unless $current_ref eq 'HASH' && exists $value->{$next};
 				return $traverser->([@$curr_path, $next], $index + 1, $value->{$next});
 			}
 		}
@@ -184,7 +182,11 @@ sub _find_field
 		return 1;    # all ok
 	};
 
-	if ($traverser->([], 0, $fields)) {
+	# manually free traverser after it's done (memroy leak)
+	my $result = $traverser->([], 0, $fields);
+	$traverser = undef;
+
+	if ($result) {
 		return [
 			map {
 				{
@@ -195,6 +197,7 @@ sub _find_field
 			} @found
 		];
 	}
+
 	return;
 }
 
@@ -206,6 +209,7 @@ sub _assign_field
 	for my $path_value (@$path_values) {
 		my @parts = @{$path_value->{path}};
 		my $current = \$fields;
+
 		for my $i (0 .. $#parts) {
 
 			# array_path will contain array indexes for each array marker
